#include <DS1307RTC.h>
#include <Time.h>
#include <TM1637Display.h>
#include <TinyWireM.h>
#include <Adafruit_NeoPixel.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>


/****************************************************************
 *                                                              *
 *  IKEA DUKTIG Kitchen upgrade  -  By Danniel Dribinski        *
 *  Date: 15/04/2020                                            *
 *  Ver: 0.1                                                    *
 *                                                              *
 *  TBD - Implement Sleep Function to save power                *
 *                                                              *
 *  Hardware: Attiny85                                          *
 *            2 LEDstrips                                       *
 *            DS1307                                            *
 *            TM1637 4 digit LED Display                        *
 *            4 buttons (+, -, start,SetClock)                  *
 *            Buzzer                                            *
 *            Door sensor                                       *
 *                                                              *
 *  Based on code by Roald Hendriks                             *
 * https://www.instructables.com/id/Kids-Kitchen-That-Says-BEEP/*
 ****************************************************************



 Revision History
  
    Date  	    By	    What
  20200415      DD      v0.1 First version, data cleanup for instructable

 */

#define NUM_LEDS 16 
#define PIN 10
Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUM_LEDS, PIN, NEO_GRB + NEO_KHZ800);



// Segments for showing BEEP
const uint8_t SEG_BEEP[] = {
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G, // B
	SEG_A | SEG_D | SEG_E | SEG_F | SEG_G,                 // E
	SEG_A | SEG_D | SEG_E | SEG_F | SEG_G,                 // E
  SEG_A | SEG_B | SEG_E | SEG_F | SEG_G,                 // P
	};


// Definition of all digital pins

// ***** Button input pins *****
// + Button - Blue
const  uint8_t  PlusButtonPin = 1;

// - Button - Green
const  uint8_t  MinusButtonPin = 2;

// Start Button - Red
const  uint8_t  StartButtonPin = 3;

// Clock - OnBoard Button
const  uint8_t  ClockButtonPin = 5;

// Door Sensor
const  uint8_t  DoorSensorPin = 7;

// ***** Output pins *****
// Buzzer
const  uint8_t  BuzzerPin = 8;

// TM1637 clock pin
const  uint8_t  TM1637CLKPin = 9;

// TM1637 data pin
const  uint8_t  TM1637DataPin = 0;



// PWM LED brightness
const  uint8_t  intLEDBrightness = 200;

// Create TM1637 object
TM1637Display display(TM1637CLKPin,TM1637DataPin);

// Create RTC time object
tmElements_t RTC_time;

// Create RTC timer objects to hold the timer
// prevTimes is used to determine whether the timer has changed
tmElements_t Timer, prevTimer;

uint8_t PlusButtonState        = LOW;    // the current  reading from the plus   input pin
uint8_t lastPlusButtonState    = LOW;    // the previous reading from the plus   input pin
uint8_t MinusButtonState       = LOW;    // the current  reading from the minus  input pin
uint8_t lastMinusButtonState   = LOW;    // the previous reading from the minus  input pin
uint8_t StartButtonState       = LOW;    // the current  reading from the start  input pin
uint8_t lastStartButtonState   = LOW;    // the previous reading from the start  input pin
uint8_t ClockButtonState       = LOW;    // the current  reading from the Clock  input pin
uint8_t lastClockButtonState   = LOW;    // the previous reading from the Clock  input pin
uint8_t DoorSensorState        = LOW;    // the current  reading from the Door   input pin
uint8_t lastDoorSensorState    = LOW;    // the previous reading from the Door   input pin


uint8_t lastPlusDebounceTime  =   0;    // the last time the plus  output pin was toggled
uint8_t lastMinusDebounceTime =   0;    // the last time the minus output pin was toggled
uint8_t lastStartDebounceTime =   0;    // the last time the start output pin was toggled
uint8_t lastClockDebounceTime =   0;    // the last time the Clock  output pin was toggled
uint8_t lastDoorDebounceTime  =   0;    // the last time the Clock  output pin was toggled


uint8_t debounceDelay         =  50;    // the debounce time in milliseconds

uint8_t intState;                         // current  state

uint8_t stripRun = 0;                   //To manage the strip color efect between cycles

// All states
#define ShowTime  1
#define SetTimer  2
#define CountDown 3
#define Beeping   4
#define SetClock  5

#define On  HIGH
#define Off LOW

time_t systime; // holds current time for diff calculation

// While setting the timer, a list with gradually increasing steps will be used
// The numbers represent seconds
uint8_t TimerSet[] = {5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60};

// For colon
uint8_t segto;

void setup(){
// Mandatory setup function   
  strip.begin();
  strip.show(); // Initialize all pixels to 'off'
  // Run white color once, to check the LEDs
  CylonBounce(0xff, 0xff, 0xff, 4, 30, 10);
  setAll(0,0,0);
  
  // Sync to DS1307
  setSyncProvider(RTC.get);
  setSyncInterval(10);
  
  // Set Timer to 00:00
  Timer.Minute = 0;
  Timer.Second = 0;
  // Set previous timer to timer
  prevTimer = Timer;
  
  // Initialize the button pins as inputs:
  pinMode(PlusButtonPin,  INPUT); 
  pinMode(MinusButtonPin, INPUT); 
  pinMode(StartButtonPin, INPUT); 
  pinMode(DoorSensorPin,  INPUT);
  
  // Initialize the output pins:
  pinMode(BuzzerPin,  OUTPUT);
  
  // Set initial mode
  intState = ShowTime;
  
  // Set default brightness to 10 on a scale of 15
  display.setBrightness(0x0a);
}


void loop(void){
// The mandatory loop
  // read the state of the minus switch into a local variable:
  uint8_t readMinusPin = digitalRead(MinusButtonPin);
  // read the state of the plus  switch into a local variable:
  uint8_t readPlusPin  = digitalRead(PlusButtonPin);
  // read the state of the start switch into a local variable:
  uint8_t readStartPin = digitalRead(StartButtonPin);
  // read the state of the Clock  switch into a local variable:
  uint8_t readClockPin  = digitalRead(ClockButtonPin);
  // read the state of the Door  Sensor into a local variable:
  uint8_t readDoorPin  = digitalRead(DoorSensorPin);

  
  
  // If the plus switch changed, due to noise or pressing:
  if (readPlusPin != lastPlusButtonState) {
    // reset the debouncing timer
    lastPlusDebounceTime = millis();
  }

  // If the minus switch changed, due to noise or pressing:
  if (readMinusPin != lastMinusButtonState) {
    // reset the debouncing timer
    lastMinusDebounceTime = millis();
  }

  // If the start switch changed, due to noise or pressing:
  if (readStartPin != lastStartButtonState) {
    // reset the debouncing timer
    lastStartDebounceTime = millis();
  }

  // If the Clock switch changed, due to noise or pressing:
  if (readClockPin != lastClockButtonState) {
    // reset the debouncing timer
    lastClockDebounceTime = millis();
  }

  // If the Clock switch changed, due to noise or pressing:
  if (readDoorPin != lastDoorSensorState) {
    // reset the debouncing timer
    lastDoorDebounceTime = millis();
  }

  

    if ((millis() - lastPlusDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer
    // than the debounce delay, so take it as the actual current state:
      if (readPlusPin == HIGH){
        switch(intState) {
          case ShowTime:
           // Change mode to SetTimer
           intState = SetTimer;
           delay(200);
           // Increase timer (from 00:00 to 00:05)
           IncreaseTimer();
           // Show timer
           ShowTimerOnDisplay();           
           break;
          case SetTimer:
           delay(200);
           // Increase the timer to smallest standard setting larger than current timer
           IncreaseTimer();
           break;
          case CountDown:
           delay(200);
           // Increase the timer to smallest standard setting larger than current timer
           IncreaseTimer();
           break;
          case SetClock:
           // Add 1 hour to the Clock
           if (RTC_time.Hour < 23){
            RTC_time.Hour=RTC_time.Hour+1;
           } else {
            RTC_time.Hour=0;
           }
           RTC.write(RTC_time);
           ShowTimeOnDisplay();
           delay(100);
           break;
        };
      }
    }
    
     if ((millis() - lastMinusDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer
    // than the debounce delay, so take it as the actual current state:
      if (readMinusPin == HIGH){
        switch(intState) {
          case ShowTime:
           break;
          case SetTimer:
           delay(200);
           // Increase the timer to largest standard setting smaller than current timer
           DecreaseTimer();
           break;
          case CountDown:
           delay(200);
           // Increase the timer to largest standard setting smaller than current timer
           DecreaseTimer();
           break;
          case SetClock:
           // Add 1 Minute to the Clock
           if (RTC_time.Minute < 59){
            RTC_time.Minute=RTC_time.Minute+1;
           } else {
            RTC_time.Minute=0;
           }
           RTC.write(RTC_time);
           ShowTimeOnDisplay();
           delay(100);
           break;
        };
      }
    }
 

    if ((millis() - lastStartDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer
    // than the debounce delay, so take it as the actual current state:
      if (readStartPin == HIGH){
        switch(intState) {
          case ShowTime:
           // Ignore
           break;
          case SetTimer:
           // Start countdown
           intState = CountDown;
           // Oven light on
           SetLight(On);
           break;
          case CountDown:
           // Show time
           intState = ShowTime;
           // Reset timer
           Timer.Minute = 0;
           Timer.Second = 0;
           break;
          case SetClock:
           // Show time
           intState = ShowTime;
           // Reset timer
           Timer.Minute = 0;
           Timer.Second = 0;
           break;
        };
      }
    }
    
     if ((millis() - lastClockDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer
    // than the debounce delay, so take it as the actual current state:

    // if the button state has changed:
    if (ClockButtonPin != ClockButtonState) {
      ClockButtonState = readClockPin;
      if (readClockPin == HIGH){
        switch(intState) {
          case ShowTime:
           // Set the Clock
           intState = SetClock;
           break;
          case SetTimer:
           // Set the Clock
           intState = SetClock;
           // Reset timer
           Timer.Minute = 0;
           Timer.Second = 0;
           break;
          case CountDown:
           // Set the Clock
           intState = SetClock;
           // Reset timer
           Timer.Minute = 0;
           Timer.Second = 0;
         case SetClock:
           // Show time
           intState = ShowTime;
           break;
        };
      };
    };
    }

    if ((millis() - lastDoorDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer
    // than the debounce delay, so take it as the actual current state:
      if (readDoorPin == HIGH){
        setAll(0,0,0);
      } else {
        //Turn on a LED in the middle of the strip
        setPixel(NUM_LEDS/2, 0xff/10, 0xff/10, 0xff/10);
        showStrip();
      }
    }


  // Based on current state, do ...
  switch(intState) {
    case ShowTime:
     // ShowTime on Display, but only when the time has changed
     if (systime != now()) { // wait for new second to do anything
        // Update systime
        systime = now();
        // ShowTime on Display
        ShowTimeOnDisplay();
     }  
     // Oven light off
     SetLight(Off);
     break;
    case SetTimer:
     // Only update display when the timer has changed
     if (Timer.Minute != prevTimer.Minute || Timer.Second != prevTimer.Second) {
       prevTimer = Timer;
       // ShowTimer on Display
       ShowTimerOnDisplay();
     }
     // Oven light off
     SetLight(Off);
     break;
    case CountDown:
     // Correct Timer, but only when the time has changed
     if (systime != now()) { // wait for new second to do anything
        systime = now();
        // ShowTimer on Display
        ShowTimerOnDisplay();
        // And of course, count down the timer
        CountdownTimer();
        //keep the leds running
        SetLight(On);
     }
     if (Timer.Minute == 0 && Timer.Second == 0) {
       // Delay so 00:01 is shown
       delay(900);
       intState = Beeping;
     }
     break;
    case Beeping:
     // Well, BEEP!!!
     Beep();
     break;
    case SetClock:
     // ShowTime on Display, 
     ShowTimeOnDisplay();
     // Oven light off
     SetLight(Off);
     break; 
  }

  lastPlusButtonState  = readPlusPin;
  lastMinusButtonState = readMinusPin;
  lastStartButtonState = readStartPin;
  lastClockButtonState = readClockPin;
  lastDoorSensorState = readClockPin;

}


void Beep(void)
// Make a beeping sound
{ // Show BEEP in display
  display.setSegments(SEG_BEEP);
  // Beep on for 500 milliseconds
  digitalWrite(BuzzerPin,HIGH);
  delay(500);
  // Beep off for 500 milliseconds
  digitalWrite(BuzzerPin,LOW);
  delay(500);
  // Beep on for 500 milliseconds
  digitalWrite(BuzzerPin,HIGH);
  delay(500);
  // Beep off for 500 milliseconds
  digitalWrite(BuzzerPin,LOW);
  delay(500);
  // Beep on for 1 second
  digitalWrite(BuzzerPin,HIGH);
  delay(1000);
  // Beep off
  digitalWrite(BuzzerPin,LOW);
  // After the beep, show time again
  intState = ShowTime;
}

void CountdownTimer()
// Decrease timer by 1 second
{
  uint8_t intSeconds;
  // Convert current timer time to seconds
  intSeconds = Timer.Minute * 60 + Timer.Second;
  
  // Decrease by 1
  intSeconds--;
  
  // Update timer
  Timer.Minute = intSeconds / 60;
  Timer.Second = intSeconds % 60;
}

void DecreaseTimer()
// Decrease timer using standard list
{ uint8_t intSeconds, intIndex;
  // Convert current timer time to seconds
  intSeconds = Timer.Minute * 60 + Timer.Second;

  // Loop through standard list and find the first value larger than the current number of seconds
  for (uint8_t i=12; i >= 0; i--){
      // Check whether list value is larger than current timer setting
      if(TimerSet[i] < intSeconds)
        { // Set Timer, minutes is simply dividing by 60, seconds are the rest after division
          Timer.Minute = TimerSet[i] / 60;
          Timer.Second = TimerSet[i] % 60;
          // Break out of for loop
          break;
        }
   } 
}

void IncreaseTimer()
// Increase timer using standard list
{ uint8_t intSeconds, intIndex;
  // Convert current timer time to seconds
  intSeconds = Timer.Minute * 60 + Timer.Second;

  // Loop through standard list and find the first value smaller than the current number of seconds
  for (int i=0; i <= 12; i++){
      // Check whether list value is smaller than current timer setting
      if(TimerSet[i] > intSeconds)
        { // Set Timer, minutes is simply dividing by 60, seconds are the rest after division
          Timer.Minute = TimerSet[i] / 60;
          Timer.Second = TimerSet[i] % 60;
          // Break out of for loop
          break;
        }
   } 
}

 
void ShowTimeOnDisplay(void)
// Show current time on the display
{ // Convert time to an integer
  RTC.read(RTC_time);
  int intNumber;
  intNumber = RTC_time.Hour * 100 + RTC_time.Minute;
  display.showNumberDec(intNumber,true);
  // For blinking the colon:
  // show colon for even seconds
  // hide colon for odd seconds
  if ((RTC_time.Second%2) == 0) {
    segto = 0x80 | display.encodeDigit((intNumber/100)%10);
    display.setSegments(&segto, 1, 1);
  }
}

void ShowTimerOnDisplay(void)
// Show timer on the display
{ // Convert timer to an integer
  uint8_t intNumber;
  intNumber = Timer.Minute * 100 + Timer.Second;
  // Show on display
  display.showNumberDec(intNumber,true);
  // Show colon
  segto = 0x80 | display.encodeDigit((intNumber/100)%10);
  display.setSegments(&segto, 1, 1);
}


void CylonBounce(byte red, byte green, byte blue, uint8_t EyeSize, uint8_t SpeedDelay, uint8_t ReturnDelay){
  //This will run "EyeSize"-LEDs along the strip and then back
  for(uint8_t i = 0; i < NUM_LEDS-EyeSize-2; i++) {
    setAll(0,0,0);
    setPixel(i, red/10, green/10, blue/10);
    for(uint8_t j = 1; j <= EyeSize; j++) {
      setPixel(i+j, red, green, blue);
    }
    setPixel(i+EyeSize+1, red/10, green/10, blue/10);
    showStrip();
    delay(SpeedDelay);
  }

  delay(ReturnDelay);

  for(uint8_t i = NUM_LEDS-EyeSize-2; i > 0; i--) {
    setAll(0,0,0);
    setPixel(i, red/10, green/10, blue/10);
    for(uint8_t j = 1; j <= EyeSize; j++) {
      setPixel(i+j, red, green, blue);
    }
    setPixel(i+EyeSize+1, red/10, green/10, blue/10);
    showStrip();
    delay(SpeedDelay);
  }
 
  delay(ReturnDelay);
}

// ***************************************
// ** FastLed/NeoPixel Common Functions **
// ***************************************

// Apply LED color changes
void showStrip() {
 #ifdef ADAFRUIT_NEOPIXEL_H 
   // NeoPixel
   strip.show();
 #endif
 #ifndef ADAFRUIT_NEOPIXEL_H
   // FastLED
   FastLED.show();
 #endif
}

// Set a LED color (not yet visible)
void setPixel(uint8_t Pixel, byte red, byte green, byte blue) {
 #ifdef ADAFRUIT_NEOPIXEL_H 
   // NeoPixel
   strip.setPixelColor(Pixel, strip.Color(red, green, blue));
 #endif
 #ifndef ADAFRUIT_NEOPIXEL_H 
   // FastLED
   leds[Pixel].r = red;
   leds[Pixel].g = green;
   leds[Pixel].b = blue;
 #endif
}


// Set all LEDs to a given color and apply it (visible)
void setAll(byte red, byte green, byte blue) {
  for(uint8_t i = 0; i < NUM_LEDS; i++ ) {
    setPixel(i, red, green, blue); 
  }
  showStrip();
}

void SetLight(boolean State)
// Set LED strip to On or Off (state HIGH or LOW)
{
  uint8_t ledSpeed = 80;
  uint8_t ledFlip = 60;
  if (State==Off) {
    //Just turn off the light
    stripRun=0;
    setAll(0,0,0);
  } else {
    //This is a heatup effect, from yellow to almost red, takes about 8 secons to see it
    switch(stripRun) {
      case 1:
        CylonBounce(0xe0, 0xff, 0x00, 4, ledSpeed, ledFlip);
        break;
      case 2:
        CylonBounce(0xcc, 0xff, 0x00, 4, ledSpeed, ledFlip);
        break;     
      case 3:
        CylonBounce(0x91, 0xff, 0x00, 4, ledSpeed, ledFlip);
        break;
      case 4:
        CylonBounce(0x7f, 0xff, 0x00, 4, ledSpeed, ledFlip);
        break;
      case 5:
        CylonBounce(0x3c, 0xff, 0x00, 4, ledSpeed, ledFlip);
        break;
      case 6:
        CylonBounce(0x14, 0xff, 0x00, 4, ledSpeed, ledFlip);
        break;
    }
   //In case the run time is longer then the efect keep the "hotest" color
   if ( stripRun < 6 ){
    stripRun++; 
   }
  }
}

void sleep() {
    GIMSK |= _BV(ACIE);                     // Enable Pin Change Interrupts
    PCMSK1 |= _BV(PCINT5);                  // Use PB3 as interrupt pin
    ADCSRA &= ~_BV(ADEN);                   // ADC off
    set_sleep_mode(SLEEP_MODE_PWR_DOWN);    // replaces above statement
    sleep_enable();                         // Sets the Sleep Enable bit in the MCUCR Register (SE BIT)
    sei();                                  // Enable interrupts
    sleep_cpu();                            // sleep
    cli();                                  // Disable interrupts
    PCMSK1 &= ~_BV(PCINT5);                  // Turn off PB3 as interrupt pin
    sleep_disable();                        // Clear SE bit
    ADCSRA |= _BV(ADEN);                    // ADC on
    sei();                                  // Enable interrupts
    } // sleep
    
ISR(PCINT0_vect) {
    // This is called when the interrupt occurs, but I don't need to do anything in it
    }
